//////////////////////////////////////////////
// main.cpp
//
//////////////////////////////////////////////

/// Includes ---------------------------------

// nkLog
#include <NilkinsLog/ConsoleLogger.h>

// nkGraphics
#include <NilkinsGraphics/Configs/ConfigHolder.h>
#include <NilkinsGraphics/Configs/ConfigManager.h>

// Included because we will want to alter the window's size
#include <NilkinsGraphics/Configs/WindowDescriptor.h>

#include <NilkinsGraphics/Log/LogManager.h>

#include <NilkinsGraphics/RenderContexts/RenderContextManager.h>

#include <NilkinsGraphics/Systems/MainSystem.h>

// nkResources
#include <NilkinsResources/ResourceManager.h>

// Standards
#include <memory>

/// Function ---------------------------------

int main ()
{
	// Prepare logging
	std::unique_ptr<nkLog::Logger> logger = std::make_unique<nkLog::ConsoleLogger>() ;
	nkGraphics::LogManager::getInstance()->setReceiver(logger.get()) ;

	// We will change a bit the configuration, to alter how the engine will initialize
	nkGraphics::ConfigHolder* config = nkGraphics::ConfigManager::getInstance()->getCurrentConfig() ;

	// We will use the Dx11 API this time (defaults to AUTOMATIC, often resulting in using Dx12)
	config->_wantedApi = nkGraphics::RENDERING_API::D3D11 ;

	config->_window.setValue(nkGraphics::WINDOW_PROPS::WIDTH, 1280) ;
	config->_window.setValue(nkGraphics::WINDOW_PROPS::HEIGHT, 1024) ;

	// This call will have no effect without the config file next to the executable
	// But it is left here for demonstration purpose
	nkGraphics::MainSystem::getInstance()->autoConfigure() ;

	// Set the path we will use to find back the data, relative to execution path
	// Now setting a resourcePath in a resource will be relative to the Data folder
	nkResources::ResourceManager::getInstance()->setWorkingPath("Data") ;

	// Init the component, using all the configuration we made up till now
	if (!nkGraphics::MainSystem::getInstance()->initialize())
		return -1 ;

	// Prepare the context and window we will use
	nkGraphics::RenderContext* context = nkGraphics::RenderContextManager::getInstance()->createRenderContextAndWindowFromConfig() ;

	// Run and clean once done
	nkGraphics::MainSystem::getInstance()->run(context) ;

	// Clean
	nkGraphics::MainSystem::getInstance()->kill() ;

	return 0 ;
}