﻿//////////////////////////////////////////////
// BlurProgramsTracker.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Program ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../Utils/ProgramTracker.h"

#include "BlurComplexEffectInfo.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>

/// Internals --------------------------------

namespace nkAstraeus
{
	struct BlurProgramsIdGenerator
	{
		static unsigned long long generate (const BlurComplexEffectInfo& info) ;
	} ;

	struct BlurProgramsNameGenerator
	{
		constexpr static const char* name = "NILKINS_BLUR_PROGRAM_" ;
	} ;

	struct BlurProgramsEntry
	{
		nkGraphics::Program* _programHorizontal = nullptr ;
		nkGraphics::Program* _programVertical = nullptr ;
		unsigned int _count = 0 ;
	} ;
}

/// Class ------------------------------------

namespace nkAstraeus
{
	class BlurProgramsTracker final : public nkCommon::SingletonClass<BlurProgramsTracker>
	{
		friend class SingletonClass<BlurProgramsTracker> ;

		private :

			// Attributes
			// Memory of programs
			std::unordered_map<unsigned long long, BlurProgramsEntry> _memory ;

		private :

			// Functions
			// Constructor, destructor
			BlurProgramsTracker () ;
			~BlurProgramsTracker () ;

		public :

			// Memory management
			std::pair<nkGraphics::Program*, nkGraphics::Program*> getPrograms (const BlurComplexEffectInfo& info) ;
			void releasePrograms (const BlurComplexEffectInfo& info) ;
	} ;
}