﻿//////////////////////////////////////////////
// RenderQueue.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Buffer ;
	class Entity ;
	class Mesh ;
	class Node ;
	class QueuedRenderableObject ;
	class Shader ;
	class SubEntity ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderQueue : public nkExport::Exportable
	{
		friend class RenderQueueManager ;

		protected :
		
			// Attributes
			// Arrays managed
			std::vector<QueuedRenderableObject*> _queuedRenderables ;
			std::vector<Entity*> _entities ;

			// Notre nom dans le cas d'une RQ speciale
			std::string _name ;
			// Ou notre index dans le cas d'une RQ normale
			unsigned int _index ;

			// Si nous sommes une queue de rendu ou pas
			bool _renderingQueue ;
			// Hidden pour les resources exports
			bool _hidden ;
			// Si le raytracing est demandé
			bool _raytraced ;

		protected :

			// Fonctions
			// Constructor; destructor
			RenderQueue () ;
			virtual ~RenderQueue () ;
		
		public :
		
			// Getters
			virtual Buffer* getAccelerationStructureBuffer () const ;
			bool getHidden () const ;

			// Setters
			void setHidden (bool value) ;

			// Sur le contrôle de la mémoire d'entity
			virtual Entity* addEntity () ;
			virtual void eraseEntity (Entity* ent) ;
			virtual void clearQueue () ;
			unsigned int getEntityCount () const ;
			Entity* getEntity (unsigned int index) const ;
			unsigned int getRenderableCount () const ;

			// Update des entity
			virtual void updateEntityNode (Entity* ent, Node* oldNode) ;
			virtual void updateAddMesh (SubEntity* subEnt) ;
			virtual void updateDeleteMesh (SubEntity* subEnt) ;
			virtual void updateSwapMesh (SubEntity* subEnt) ;
			virtual void updateSwapShader (Entity* ent) ;
			virtual void updateSwapShaderRaytracing (Entity* ent, Shader* oldShader, Shader* newShader) ;

			// Opérations avec les autres RQ
			void appendQueue (RenderQueue* other) ;

			// Nom
			virtual void setName (const nkMemory::StringView& name) ;
			nkMemory::StringView getName () const ;

			// Index
			void setIndex (unsigned int index) ;
			unsigned int getIndex () const ;

			// Rendering queue
			void setRenderingQueue (bool value) ;
			bool getRenderQueue () const ;

			// Raytracing
			virtual void setRaytraced (bool value) ;
			bool getRaytraced () const ;

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}