﻿//////////////////////////////////////////////
// GlesRenderContext.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

#if defined(_WIN32)
	typedef struct HGLRC__* HGLRC ;
	typedef struct HDC__* HDC ;
#endif

/// Includes ---------------------------------

// nkGraphics
#include "../../../RenderContexts/RenderContext.h"

/// Class ------------------------------------

namespace nkGraphics::gles
{
	class GlesRenderContext : public RenderContext
	{
		private :

			// Attributes
			#if defined(_WIN32)
				HGLRC _context ;
				HDC _hdc ;
			#elif defined(__EMSCRIPTEN__)
				int _context ;
			#endif

			// Gamma correction activation
			bool _gammaCorrected ;

		private :

			// Functions
			// Creation
			void _createContextOffscreen (const RenderContextDescriptor& descriptor) ;
			void _createContextWindow (const RenderContextDescriptor& descriptor) ;

		public :

			// Constructor, destructor
			GlesRenderContext () ;
			~GlesRenderContext () ;

			// Getters
			#if defined(_WIN32)
				HGLRC getContext () const ;
				HDC getHdc () const ;
			#elif defined(__EMSCRIPTEN__)
				int getContext () const ;
			#endif

			// Load
			virtual void initialize (const RenderContextDescriptor& descriptor) override ;

			// Rendering
			virtual void swap () override ;
			void activate () ;
			
			// Chain manipulation
			virtual void refreshBufferSize (unsigned int width = 0, unsigned int height = 0) override ;
			virtual void refreshViewportSize () override ;
			virtual void setGammaCorrected (bool value) override ;
	} ;
}